#include <math.h>

/* Please note that the following functions need to be prototyped correctly */
/* before you begin to use them                                             */




/* This function locates a point given the command 1. The input required */
/* is the point number (count), the reference point number (index), 
/* the distance from the reference point (length) and the angle made at the */
/* reference point (alpha).   The data arrays x and y are used to store the  */
/* values of the x and y coordinates of each point.                   */

void locate(int count, int index,double x[], double y[], double length, double alpha)
{
  
  x[count] = x[index] + length * cos(alpha) ;
  y[count] = y[index] + length * sin(alpha) ;
}


/* This function locates a point given the command 2. The input required  */
/* is the point number (count), the reference point numbers(index1 & index2)*/
/* and the angles alpha and beta made at the two points index1 and index2 */
/*  respectively. Again the coordinates are stored in the arrays x and y  */  

void intersection(double x[], double y[],int index1, int index2, 
                  double alpha, double beta, int count)
{                        
  double m1, m2,x1,y1,x2,y2 ;

  m1 = tan(alpha) ;         /* m1 and m2 are the slopes of the two lines at */
  m2 = tan(beta)  ;         /* the two points.                               */
  x1 = x[index1]  ;
  x2 = x[index2]  ;
  y1 = y[index1]  ;
  y2 = y[index2]  ;

  if(m1 < 1.0e+5) {      /* Checking to see that the slopes are not infinite */
    if(m2 < 1.0e+5) {
      x[count] = ((m1*x1 - m2*x2) + y2-y1) / (m1-m2) ;
      y[count] = ((m1 * m2)*(x1-x2) + (m1*y2-m2*y1)) / (m1-m2)  ;
    }
    else {
      x[count] = x2 ;
      y[count] = m1*(x2-x1) + y1 ;
    }
  }
    else{ 
      x[count] = x1 ;
      y[count] = m2*(x1-x2) + y2 ;
    }
}

/* The following function computes the length of a side given the coordinates*/
/*  of the endpoints                             */

double side_length(double x1, double x2,double y1, double y2)
{
  double length ;
  length = sqrt ((x2-x1)*(x2-x1) + (y2-y1)*(y2-y1) ) ;         
  return(length) ;
}

/* The following function returns the area of a triangle. The input is the */
/* length of each of the three sides of the triangle (a,b,c). This function */
/* may be used to compute the area of a convex polygon.                     */

double area_triangle(double a , double b , double c)
{

  double area, s ;

  s = (a+b+c) /2.0 ;
  area = sqrt( s * (s-a) * (s-b) * (s-c)) ;
  return (area) ;

}
 
