#include <stdio.h>

/*
 * These are prototypes for the functions provided for us in Q2_code.c.
 */
extern void locate (int, int, double [], double [], double, double);
extern void intersection(double [], double [], int, int, double, double, int);
extern double side_length(double, double, double, double);
extern double area_triangle(double, double, double);

/*
 * Here are the arrays the points will be stored in.
 */
double x_points[10], y_points[10];

/*
 * This function prints a prompt and gets an int from the standard
 * input.  It repeats until a valid int is entered./
 */
int getint(const char *prompt)
{
     int ret, assigned;
     
     do {
	  printf("%s: ", prompt);
	  assigned = scanf(" %d", &ret);
	  if (assigned != 1) {
	       printf("Bad input!\n");
	  }
     } while (assigned != 1);

     return(ret);
}

/*
 * Same thing as getint, but gets a double, not an int.
 */
double getdouble(const char *prompt)
{
     int assigned;
     double ret;
     
     do {
	  printf("%s: ", prompt);
	  assigned = scanf(" %lf", &ret);
	  if (assigned != 1) {
	       printf("Bad input!\n");
	       continue;
	  }
	  break;
     } while (1);

     return(ret);
}

/*
 * Same thing as getint, but requires the int to be between 1 and 10.
 */
int getpoint(const char *prompt)
{
     int point;
     
     do {
	  point = getint(prompt);
	  if ((point > 10) || (point < 1)) {
	       printf("Must be between 1 and 10!\n");
	       continue;
	  }
	  break;
     } while (1);

     return(point - 1);
}

main()
{
     int command;
     int point, point2, point3;
     double distance, angle, angle2, area;
     double side1, side2, side3;
     int i, numpoints;

     for (i = 0; i < 10; i++) {
	  x_points[i] = y_points[i] = 0.0;
     }
     
     while (1) {
	  command = getint("Enter command");
	  switch (command) {
	  case 0:
	       /* Direct coordinate assignment */
	       point = getpoint("Enter point to be assigned");
	       x_points[point] = getdouble("Enter x coordinate");
	       y_points[point] = getdouble("Enter y coordinate");
	       printf("Point %d set to (%f, %f).\n", point + 1,
		      x_points[point], y_points[point]);
	       break;
	  case 1:
	       /* Angle-distance assignment */
	       point = getpoint("Enter point to be assigned");
	       point2 = getpoint("Enter point from which to measure");
	       distance = getdouble("Enter distance");
	       angle = getdouble("Enter angle");
	       locate(point, point2, x_points, y_points, distance, angle);
	       printf("Point %d set to (%f, %f).\n", point + 1,
		      x_points[point], y_points[point]);
	       break;
	  case 2:
	       /* Intersection assignment */
	       point = getpoint("Enter point to be assigned");
	       point2 = getpoint("Enter first reference point");
	       point3 = getpoint("Enter second reference point");
	       angle = getdouble("Enter first reference angle");
	       angle2 = getdouble("Enter second reference angle");
	       intersection(x_points, y_points, point2, point3, angle, angle2,
			    point);
	       printf("Point %d set to (%f, %f).\n", point + 1,
		      x_points[point], y_points[point]);
	       break;
	  case 3:
	       /* Calculate distances */
	       point = getpoint("Enter first point");
	       point2 = getpoint("Enter second point");
	       printf("The distance between the points %d and %d was found to be %.6f\n",
		      point + 1, point2 + 1,
		      side_length(x_points[point], x_points[point2],
				  y_points[point], y_points[point2]));
	       break;
	  case 4:
	       /* Calculate convex polygon area */
	       do {
		    numpoints = getint("Enter number of points");
		    if (numpoints < 3) {
			 printf("Must have at least three points!\n");
			 continue;
		    }
		    break;
	       } while (1);
	       /*
		* The first two points are assigned before the loop
		* starts.  Then, each time through the loop, one new
		* point is retrieved in order to complete the
		* triangle, the area of that triangle is calculated,
		* and then one of points and sides is thrown away (and
		* replaced by the most recent one) to make room for
		* the next point and side.
		*/
	       point = getpoint("Enter first point");
	       point2 = getpoint("Enter next point");
	       side1 = side_length(x_points[point], x_points[point2],
				   y_points[point], y_points[point2]);
	       area = 0.0;
	       i = numpoints - 2;
	       while (i--) {
		    point3 = getpoint("Enter next point");

		    side2 = side_length(x_points[point], x_points[point3],
					y_points[point], y_points[point3]);
		    side3 = side_length(x_points[point2], x_points[point3],
					y_points[point2], y_points[point3]);

		    area += area_triangle(side1, side2, side3);

		    point2 = point3;
		    side1 = side2;
	       }
	       printf("Total area is %.6f\n", area);
	       break;
	  case 5:
	       /* print table of points */
	       printf("\nPOINT    X                Y\n\n");
	       for (i = 0; i < 10; i++) {
		    printf("   %-2d   %-16.6f%-16.6f\n", i + 1, x_points[i],
			   y_points[i]);
	       }
	       break;
	  case 100:
	       printf("Thanks for using the program !\n");
	       exit(0);
	       /* NOTREACHED */
	       break;
	  default:
	       printf("Unknown command!\n");
	       break;
	  }
     }
}
