#include <matrix.h>
#include <math.h>
#include <float.h>
#include <stdio.h>

#define TOLERANCE	0.005
#define MASS 		4000.0
#define STIFFNESS	4000000.0
#define PI		3.14159265358979323846


extern Matrix mass_matrix(int floors, double mass);
extern Matrix stiffness_matrix(int floors, double stiffness);
extern Matrix inverse_stiffness_matrix(int floors, double stiffness);


double matrix_max(Matrix matrix)
{
     int i, j, maxi, maxj;
     double max = -DBL_MAX, new;

     maxi = matrix_rows(matrix);
     maxj = matrix_columns(matrix);

     for (i = 1; i <= maxi; i++) {
	  for (j = 1; j <= maxj; j++) {
	       new = matrix_get(matrix, i, j);
	       if (new > max)
		    max = new;
	  }
     }

     return(max);
}


main()
{
     int floors, ret;
     Matrix M, K_inverse, x1 = 0, q = 0, Phi1 = 0;
     double omega = -TOLERANCE, old_omega;

     do {
	  printf("How many floors are there? ");
	  ret = scanf(" %d", &floors);
     } while ((ret != 1) && (ret >= 0));

     if (ret < 0) /* EOF */
	  exit(0);
     
     if (floors < 1) {
	  fprintf(stderr, "Must have at least 1 floor!\n");
	  exit(1);
     }
     else if (floors > 10) {
	  fprintf(stderr, "Can't have more than 10 floors!\n");
	  exit(1);
     }
     
     M = mass_matrix(floors, MASS);
     K_inverse = inverse_stiffness_matrix(floors, STIFFNESS);

     x1 = matrix_create(floors, 1);
     matrix_fill(x1, 1.0);

     do {
	  old_omega = omega;

	  matrix_multiply(M, x1, &q);

	  matrix_multiply(K_inverse, q, &Phi1);

	  omega = matrix_max(x1) / matrix_max(Phi1);

	  matrix_scale(Phi1, omega, &x1);
     } while (fabs(omega - old_omega) > TOLERANCE);

     printf("The natural frequencies of the building are multiples of %f Hertz.\n",
	    sqrt(omega) / 2.0 / PI);

     exit(0);
}

