#include "pset6.h"

static NODE read_nodes(FILE *node_input, int *n_nodes)
{
     NODE nodes;
     int i;
     int node_number;
     double x, y;
     
     if (fscanf(node_input, " %d", n_nodes) != 1) {
	  fprintf(stderr, "Error reading number of nodes from node input.\n");
	  return(0);
     }

     nodes = (NODE) calloc(*n_nodes, sizeof(*nodes));
     if (! nodes) {
	  fprintf(stderr, "Error allocating space for nodes.\n");
	  return(0);
     }

     /*
      * Initialize all the elements of the array to something
      * detectable so that we can check after reading the data file to
      * make sure all nodes were in the file.
      */
     for (i = 0; i < *n_nodes; i++) {
	  nodes[i].node_number = UNVISITED;
	  nodes[i].x = nodes[i].y = 0.0;
	  nodes[i].first = 0;
     }

     for (i = 0; i < *n_nodes; i++) {
	  if (fscanf(node_input, " %d %F %F", &node_number, &x, &y) != 3) {
	       fprintf(stderr, "Error reading node from node input.\n");
	       return(0);
	  }
	  if ((node_number < 1) || (node_number > *n_nodes)) {
	       fprintf(stderr, "Illegal node number %d in node input.\n",
		       node_number);
	       return(0);
	  }
	  if (nodes[node_number-1].node_number != UNVISITED) {
	       fprintf(stderr, "Duplicate node %d in node input.\n",
		       node_number);
	       return(0);
	  }
	  nodes[node_number-1].node_number = node_number;
	  nodes[node_number-1].x = x;
	  nodes[node_number-1].y = y;
     }

     /*
      * Given the error checking inside the loop above, it should not
      * be possible for there to be any problems with the data at this
      * point, but we do a quick check anyways just to be safe.
      */
     for (i = 0; i < *n_nodes; i++) {
	  if (nodes[i].node_number == UNVISITED) {
	       fprintf(stderr, "Node %d not specified in node input.\n", i);
	       return(0);
	  }
     }

     return(nodes);
}

		       
static NODE read_streets(FILE *street_input, NODE nodes, int n_nodes)
{
     struct arc new;
     int street_number, origin, end;
     double length, cost;
     
     while (fscanf(street_input, " %d %d %d %F %F", &street_number, &origin,
		   &end, &length, &cost) == 5) {
	  if ((origin < 1) || (origin> n_nodes)) {
	       fprintf(stderr,
		       "Illegal origin node %d for street %d in street input.\n",
		       origin, street_number);
	       return(0);
	  }
	  if ((end < 1) || (end > n_nodes)) {
	       fprintf(stderr,
		       "Illegal end node %d for street %d in street input.\n",
		       end, street_number);
	       return(0);
	  }
	  new.street = street_number;
	  new.connects_to = origin;
	  new.length = length;
	  new.cost = cost;
	  if (! (nodes[end-1].first =
		 insert_arc(nodes[end-1].first, new))) {
	       return(0);
	  }
	  new.connects_to = end;
	  if (! (nodes[origin-1].first =
		 insert_arc(nodes[origin-1].first, new))) {
	       return(0);
	  }
     }
     if (! feof(street_input)) {
	  fprintf(stderr, "Error reading from street input.\n");
	  return(0);
     }

     return(nodes);
}


	 
NODE read_zone(FILE *node_input, FILE *street_input, int *n_nodes)
{
     NODE nodes;

     nodes = read_nodes(node_input, n_nodes);
     if (! nodes)
	  return(0);
     
     nodes = read_streets(street_input, nodes, *n_nodes);
     if (! nodes)
	  return(0);

     return(nodes);
}
