#include "pset6.h"

#define VISITED (1<<0)
#define IN_STACK (1<<1)

#define CHECK(a) if (! (a)) { fprintf(stderr, "Error allocating memory in do_routing.\n"); return; }
			       
static void do_routing(NODE nodes, int n_nodes, double **_label, int **_dad,
		       double **_cost, int base_node)
{
     char *flags;
     int i;
     double *label, *cost;
     int *dad;

     *_label = *_cost = 0;
     *_dad = 0;
     
     CHECK(label = (double *) calloc(n_nodes, sizeof(*label)));
     CHECK(dad = (int *) calloc(n_nodes, sizeof(*dad)));
     CHECK(cost = (double *) calloc(n_nodes, sizeof(*cost)));
     CHECK(flags = (char *) calloc(n_nodes, sizeof(*flags)));

     if ((base_node < 1) || (base_node > n_nodes)) {
	  fprintf(stderr, "Invalid base_node in do_routing.\n");
	  return;
     }
	  
     for (i = 0; i < n_nodes; i++) {
	  label[i] = UNVISITED;
	  dad[i] = 0;
	  cost[i] = 0;
	  flags[i] = 0;
     }

     if (stack_push(base_node) < 0)
	  return;
     flags[base_node-1] = IN_STACK & VISITED;
     label[base_node-1] = 0.0;
     
     while (! stack_empty()) {
	  int i;
	  ARC j;

	  if ((i = stack_pop()) < 0)
	       return;
	  flags[i-1] |= ~IN_STACK;

	  j = nodes[i-1].first;

	  while (j) {
	       if (label[j->connects_to-1] > label[i-1] + j->length) {
		    dad[j->connects_to-1] = i;
		    label[j->connects_to-1] = label[i-1] + j->length;
		    cost[j->connects_to-1] = cost[i-1] + j->cost;
		    if (! (flags[j->connects_to-1] & IN_STACK)) {
			 if (flags[j->connects_to-1] & VISITED)
			      if (stack_insert(j->connects_to) < 0)
				   return;
			 else
			      if (stack_push(j->connects_to) < 0)
				   return;
			 flags[j->connects_to-1] = IN_STACK & VISITED;
		    }
	       }
	       j = j->next;
	  }
     }

     *_label = label;
     *_dad = dad;
     *_cost = cost;

     free((void *) flags);

     return;
}


static void print_each(int base_node, int the_node, double label, double cost,
		       int *dad, FILE *output)
{
     int i, bool;

     if ((dad[the_node-1] == 0) && (the_node != base_node)) {
	  fprintf(output, "Shortest route to node %d: NOT CONNECTED\nlength of shortest route to node %d: NOT APPLICABLE\ncost of shortest route to node %d:$ NOT APPLICABLE\n\n", the_node, the_node, the_node);
	  return;
     }
     
     fprintf(output, "Shortest route to node %d: ", the_node);

     for (i = the_node; i; i = dad[i-1])
	  if (stack_push(i) < 0)
	       return;

     do {
	  if ((i = stack_pop()) < 0)
	       return;
	  bool = stack_empty();
	  fprintf(output, "%d%s", i, bool ? "\n" : ", ");
     } while (! bool);
     
     fprintf(output, "length of shortest route to node %d: %.2f miles\ncost of shortest route to node %d:$ %.2f\n\n", the_node, label, the_node, cost);

     return;
}


void print_routing(NODE nodes, int n_nodes, int base_node, FILE *output)
{
     double *label, *cost;
     int *dad;
     int i;
     
     do_routing(nodes, n_nodes, &label, &dad, &cost, base_node);
     if (! (label && cost && dad))
	  return;
	  
     fprintf(output, "starting node = %d\n\n", base_node);

     for (i = 0; i < n_nodes; i++)
	  print_each(base_node, nodes[i].node_number, label[i], cost[i],
		     dad, output);

     free((void *) label);
     free((void *) cost);
     free((void *) dad);
     
     return;
}
