#include <stdlib.h>
#include <stdio.h>
#include <strings.h>
#include "section.h"
#include "tree_stuff.h"

#define NEW(x) ((x *) malloc(sizeof(x)))



/*
 * Given a designation, area, depth, and Ixx, return a pointer to a
 * struct section allocated and initialized with the given data.
 */
struct section *make_node(char design[], double area, double depth,
			  double Ixx)
{
     struct section *new;

     new = NEW(struct section);
     if (! new) {
	  fprintf(stderr, "Error allocating memory in make_node.\n");
	  exit(1);
     }

     strncpy(new->designation, design, 30);
     new->designation[29] = '\0';
     new->area = area;
     new->depth = depth;
     new->Ixx = Ixx;
     new->left = new->right = new->next = 0;

     return(new);
}



/*
 * Given two nodes, return a negative number if the first one is "less
 * than" the second one, a positive number if the first one is
 * "greater than" the second one, or 0 if they are "equal".
 */
static int nodecmp(struct section *node1, struct section *node2)
{
     if (node1->Ixx < node2->Ixx)
	  return(-2);
     else if (node1->Ixx > node2->Ixx)
	  return(2);
     else if (node1->area < node2->area)
	  return(-1);
     else if (node1->area > node2->area)
	  return(1);
     else
	  return(0);
}



/*
 * Given a sorted binary tree and a new node, and the node to the
 * binary tree.
 */
void add_node(struct section *root, struct section *node)
{
     if (nodecmp(node, root) < 0)
	  if (root->left)
	       add_node(root->left, node);
	  else
	       root->left = node;
     else
	  if (root->right)
	       add_node(root->right, node);
	  else	       
	       root->right = node;
     return;
}



/*
 * Given a sorted list and a new node, add the node to the list and
 * return the new list.
 */
struct section *add_list(struct section *list, struct section *node)
{
     struct section *last, *current;

     /*
      * Special case if new node should appear at beginning of list
      */
     if ((! list) || (node->area < list->area)) {
	  node->next = list;
	  return(node);
     }

     for (last = list, current = list->next;
	  current && (current->area < node->area);
	  last = current, current = current->next) /* empty */;

     last->next = node;
     node->next = current;

     return(list);
}



/*
 * Given a tree and a moment of inertia as input, return a list of at
 * most ten sections with the given or greater moment of inertia, and
 * the number of sections in the list being returned.
 *
 * A comment about this function:
 * 
 * The specification for part B4 says to use Ixx > Igiven as the
 * criterion.  However, it seems to me that in this application, it
 * would be more appropriate to use Ixx >= Igiven, since the person
 * running the problem is obviously specifying the minimum required
 * moment of inertia.  Therefore, that is what I will use below.  I
 * could do things the other way just by replacing the < comparison
 * below with <=.
 */
struct section *find_section(struct section *root, struct section *list,
			     int *count, double Ig)
{
     if (! root)
	  return(list);

     /*
      * If this node is less than Ig, everything to the left must also
      * be less than Ig.
      */
     if (root->Ixx < Ig) {
	  return(find_section(root->right, list, count, Ig));
     }

     list = find_section(root->left, list, count, Ig);
     if (*count == 10)
	  return(list);

     list = add_list(list, root);
     (*count)++;
     if (*count == 10)
	  return(list);

     return(find_section(root->right, list, count, Ig));
}



#ifdef DEBUG
void print_tree(struct section *root)
{
     if (! root)
	  return;

     print_tree(root->left);

     printf("%s\t%f\t%f\t%f\n", root->designation, root->area,
	    root->depth, root->Ixx);

     print_tree(root->right);

     return;
}
#endif /* DEBUG */

