#include <stdio.h>
#include <stdlib.h>
#include "section.h"
#include "prototypes.h"



void file_error(FILE *file, char *message)
{
     fprintf(stderr, "%s %s.\n", ferror(file) ? "Read error" :
	     (feof(file) ? "Premature end-of-file" : "Bad format"), message);
     exit(1);
}



void mem_error(char *message)
{
     fprintf(stderr, "Out of memory %s.\n", message);
     exit(2);
}



/*
 * Read the section list from a data file and return an array of
 * pointers to Xsection structures containing the data from the file.
 * The "in" argument is the file to read the data from.  The
 * "num_sections" argument is a return parameter in which the number
 * of sections read is stored before the function returns.
 */
XSECTION *read_data(FILE *in, int *num_sections)
{
     int i;
     XSECTION datablock;
     XSECTION *ptrblock;

     if (fscanf(in, " %d", num_sections) != 1)
	  file_error(in, "while reading num_sections from data file");

     datablock = CALLOC(*num_sections, struct Xsection);
     if (! datablock)
	  mem_error("while allocating space for datablock");

     ptrblock = CALLOC(*num_sections, XSECTION);
     if (! ptrblock)
	  mem_error("while allocating space for ptrblock");

     for (i = 0; i < *num_sections; i++) {
	  if (fscanf(in, " %s %F %F %F", datablock[i].designation,
		     &datablock[i].area, &datablock[i].depth,
		     &datablock[i].Ixx) != 4)
	       file_error(in, "while reading section data from data file");
	  datablock[i].count = 0;
	  ptrblock[i] = &datablock[i];
     }

     return(ptrblock);
}

