#include <stdio.h>
#include <X11/Xlib.h>
#include <X11/Xutil.h>
				/* yes they are all globals! */
Display *display;          /* pointer to the display                 */
Window win;                /* window to be used for graphics         */
int screen_num;            /* screen number                          */
Window root;               /* root window of display                 */
XSizeHints size_hints;     /* size hint structure for window manager */
GC the_GC;                 /* graphics context to be used            */
GC copyGC;                 /* main GC                                */ 
XGCValues the_GC_values;   /* values in graphics context             */
Screen *screen_point;      /* pointer to screen                      */
			   /* Removed FILE declarations              */
Pixmap buff;               /* Pixmap used as drawing buffer          */
XEvent event;              /* structure to hold an XEvent            */


#define INIT_NAME "start_graphics"     /* name of startup module */

#define E_NOOPEN    -1		/* error--unable to open X connection */
#define E_NOFILE    -2		/* error--unable to open data file    */
#define E_NOEVENTS  -3		/* error--no events in file           */
#define BORDER 2		/* size of window border              */
#define void int
#define TRUE  1
#define FALSE 0

/*-------------------------------------------------------------------------*/
/*  This routine starts up the X connection, creates and maps a window,
    creates a Pixmap which gets returned to the caller and generally
    initializes things for the 1.00 student. The display pointer and the 
    graphics context are returned to the user through pointer arguments */

Pixmap start_graphics(width,height,p_display,p_context)
     unsigned int width;       /* this is the width of window */
     unsigned int height;      /* this is the height of window */
     Display **p_display;      /* address of display pointer(returned) */
     GC *p_context;            /* pointer to graphics context(returned) */
{

				/* Open display and set key variables */

  if((display=XOpenDisplay(NULL)) == NULL)  {
    fprintf(stderr,"%s: can't open display",INIT_NAME);
    exit(E_NOOPEN);
  }
  screen_num = DefaultScreen(display);
  root = DefaultRootWindow(display);
  screen_point = XScreenOfDisplay(display,screen_num);

  SetSizeHints(width,height);

				/* create the window */
  win = XCreateSimpleWindow(display,root,size_hints.x,size_hints.y,
			    size_hints.width, size_hints.height,BORDER,
			    BlackPixel(display,screen_num),
			    WhitePixel(display,screen_num));

				/* set properties for window manager */
  XSetStandardProperties(display,win,"GraphicsTool","1.00 Problem Sets", None,
			 0,NULL,&size_hints);

				/* elect events of interest and map window */
  XSelectInput(display,win,ExposureMask|ButtonPressMask|ButtonReleaseMask);
  
				/* setup GC */
  SetupGC();
  XMapWindow(display,win);

				/* create blanked out Pixmap buffer for user */
  buff = XCreatePixmap(display,win,width,height,
		       DefaultDepthOfScreen(screen_point));

  ClearPixmap(width, height);
  XSynchronize(display, TRUE); /* Allow other processes to share
				     CPU                            */
				/* load return values */
  *p_display = display;
  *p_context = the_GC;
  return(buff); 
}

ClearPixmap(w, h)
int w ,h;
{
  GC gc;
  
  gc = XCreateGC(display, win, None, &the_GC_values);
  XSetGraphicsExposures(display, gc, False);
  XSetForeground(display, gc, WhitePixel(display, screen_num));
  XFillRectangle(display, buff, gc, 0, 0, w,  h);
}

				/* this routine handles XEvents */
				/* This is changed so that the  */
				/* function now requires that   */
				/* a pointer to a XButtonEvent  */
				/* is passed as an argument and */
				/* loaded with a call to        */
				/* XNextEvent.                  */

void get_mouse_event(XButtonEvent * event)
{				/* Replaced: 4/91 ROC           */
				/* XButtonEvent *xb_event=NULL; */
				/* with XEvent an_event; so     */
				/* entire event struct can be   */
				/* used.                        */
  XEvent an_event;		
				/* set up event pointer for mouse */
				/* events check for XEvents to be */
				/* handled or returned to caller  */

				/* Removed update graphics call as     */
				/* it has nothing to do with the event */
				/* handler call. It is still called if */
				/* an expose event occurs.             */
				/* update_graphics();                  */
  
  short int button = FALSE;	/* control var to skip over non-button */
				/* events.                             */

  while(!button) {		     /* Wait till a button is pressed */
    XNextEvent(display, &an_event);  /* Blocks until event occurs     */
    
    switch(an_event.type){	/* Select event of interest.          */
      case(ButtonPress):
	case(ButtonRelease):{
	  *event = an_event.xbutton; /* load xbutton struct into      */
				     /* callers XButtonEvent struct   */
	  button = TRUE;	     /* allow exit from loop.         */
	} 
      break;
				/* if window obscured and then exposed */
				/* redraw contents of buffer to screen */
      case(Expose):{		
	update_graphics();
      } 
      break;
    default:{
    }
      break;    
    }
  }
}

void update_graphics()
{
  XCopyArea(display,buff,win,copyGC,0,0,
	  size_hints.width,size_hints.height,0,0);
}

SetSizeHints(width,height)
int width, height;
{
  size_hints.x = 200;   
  size_hints.y = 50;
  size_hints.min_width = width;
  size_hints.min_height = height;
  size_hints.max_width = width; 
  size_hints.max_height = height;
  size_hints.width = width;
  size_hints.height = height;
  size_hints.flags = USSize|USPosition|PMinSize|PMaxSize;
}

SetupGC()
{
  the_GC_values.foreground = BlackPixel(display, screen_num);
  the_GC_values.function = GXequiv;  /* this is for black on white */

  the_GC = XCreateGC(display,win,
		     (GCFunction | GCForeground),
		     &the_GC_values);

  copyGC = XCreateGC(display, win, None, &the_GC_values);

  XSetGraphicsExposures(display, the_GC, False);
  XSetGraphicsExposures(display, copyGC, False);
}

