@Device(postscript)
@libraryfile(accents)
@style(fontfamily Timesroman, size 14, spacing 2.2,
leftmargin 1 inches, rightmargin 1 inches, topmargin 1 inches,
bottommargin 1 inches, hyphenation on, justification on)
@Set(page=0)
@blankspace(3 inches)
@Majorheading(Ravel's @i[Bolero]:)
@Heading(A formal and periodic analysis)

@blankspace(2 inches)

@begin(center)
by Jonathan Kamens
@blankspace(0.25 inch)
May 11, 1988
@End(center)
@Newpage()
@Pageheading(Center "@Value(page)", immediate)
@Heading(Introduction)

When a person is examining a form of music to which he has not yet
been thoroughly exposed, several goals are implicit in that
examination.  In other words, what do I want to get out of a kind of
music when I listen to it for the first time?  The answer is
simple.... I want to be able to recognize what makes the music
different from other types which I've heard, and I want to be able to
evaluate what I feel and figure out whether I like what I'm hearing or
not.  I would like to present in this paper my impressions of Ravel's
@i[Bolero], a piece which presents a musical "form" which is not quite
a form, and which is not quite like anything we've done in class
before.

@Heading(The Composer)

Maurice Ravel is not as easy to categorize as a composer as perhaps
Bach or Beethoven might be.  He does not follow a strict style, like
Bach's Baroque or Beethoven's Classical.  Of course, we know that Bach
and Beethoven both made innovations which proved that they were not
limited to strict and Baroque and strict Classical forms of
composition respectively, but they were, and will always be seen as, a
Baroque and a Claassical composer.  Ravel, however, is different.
Born in 1875, he came into his own at the very end of the Romantic
era and ushered in the music of the twentieth century.  He therefore
draws on some Romantic styles in his work, but he also utilizes a
large variety of Classical techniques.@Foot[Schindler, Allan.
@u{Listening to Music}.  New York: Holt, Rinehart and Winston (1980),
p. 238.]

Ravel was born in a fishing village on the border between France and
Spain, then son of a Basque-descended mother and a Swiss father.
Although his lineage does not imply the "true Parisienne," his family
moved to Paris when he was very little and he spent most of his life
there.  In particular, his father was eager to encourage his son to
pursue a career in music as soon as the talent for it was revealed.
He studied with many well-known teachers, until 1889 when he entered
the Conservatoire of Paris.  He studied harmony (from the point of
view of a performer more than of a composer -- he was known as an
accomplished pianist) there until 1895, then left for over a year.  He
returned in 1897 to study composition.

Although he was learning much at the Conservatoire, it was not the
type of learning which the teachers there had in mind.  His music was
different, revolutionary, and it did not follow the rules of Romantic
Composition which the students were obliged to obey.  As a result,
Ravel failed to win one of the prizes necessary to stay at the
Conservatoire.  He was allowed to stay on for some time as an
"auditeur,"@Foot(Sadie, Stanley ed.  @u[The New Grove Dictionary of
Music and Musicians].  London: Macmillan Publishers Limited (1980).
vol. 15, p. 611) but in 1903 this welcome, too, expired and he was
forced to leave the Conservatoire, despite the fact that by this time
several pieces of his had been published.

By 1905, he was nearing that status of firmly-established composer in
his native Paris and in much of the rest of Europe as well.  He once
again attempted to earn the prize which would earn him a place in the
Conservatoire -- the Prix de Rome.  Once again, he was passed over in
favor of students who correctly followed the guidelines of the first
level of composition required from each competitor.  This created
quite a stir, since the Conservatoire had publicly rejected a known
composer.  As a result, the director of the Conservatoire eventually
was forced to resign, and a more liberal group came into power.  But
Ravel still did not earn his Prix de Rome.

Between 1905 and the start of World War I, Ravel began to do some of
his most innovative composing.  His new styles began to develop, and
he began to write compositions that were more mature in outlook.  One
of the ideas which he developed is that of the "musical object," an
independent piece of music which can survive on its own or be used as
a building block for more complex compositions.  his @i[Bolero],
discussed in this paper, is a perfect example of the musical object
style of composition, but that will be discussed in more detail later.

World War I marked a period of trial for Ravel.  He wanted to serve in
the army in his country's defense, but he was refused admittance into
the army because of his lower than average height and weight.  He then
attempted to earn a place for himself in the air force, but that, too,
was denied him, and he was left as a driver in the motor transport
corps.  Eventually he became sick, in part possibly because he missed
his composing, and was sent back to Paris to recover.  It was with
extreme sadness that he arrived home just barely before his mother
died.

During this period, Ravel did not explore many new ideas.  Instead, he
turned back and chose to work with older Romantic and Classical ideas.
For example, it was during this period that he composed his famous
orchestration of Mussorgsky's @i[Pictures at an Exhibition].

Eventually, Ravel overcame his disappointment and began to grow in his
compositional style again.  By 1928, when he made a trip to the United
States, he was known all over the world.  It was during this trip that
he was awarded an honorary degree by Oxford University, showing how
much the world had come to appreciate his music during his lifetime.

Unfortunately, Ravel's health had slowly been dwindling since the war.
He eventually reached the stage where he was no longer able to even
write his name, due to an illness known as "Pick's Disease," which
eventually left him helpless.  He died in 1937.

@Heading(The Music)

The particular Ravel piece which I examine here, his @i[Bolero], is
different in several important ways from any music we've studied
before.  First of all, the instrumentation is astonishingly varied....
flutes, oboes, coronets, clarinets, bassoons, trumpets, a trombone,
a tuba, saxophones, a harp, tambourines, cymbals and a snare drum, and
of course the standard violins, violas, cellos and basses make up the
orchestra.  In no other period up until now have we seen so bold of a
mix of instruments.

Furthermore, we have studied the rondo form and various other theme
and variations forms, but we have never encountered a piece in which
the theme stays in a @i[completely identical melodic line throughout
the piece].  Is is as if Ravel scoffs at hundreds of years of musical
history and says, "Melody?  I don't need to use Melody!  Give me a
melody and I can transform it into a masterpiece without changing one
note!"  He takes the incredible range of instrumental sounds, textures
and dynamics he has given himself, and he builds with these tools what
is in my opinion an absolutely incredible piece of music.

The piece itself is incredibly simple to describe, and this is part of
its beauty.  At the entrance, we have the violas and cellos playing
pizzacato along with a tambourine accompaniment which remains constant
in at least one voice throughout the piece.  This rhythmic
accompaniment, seen in Figure A attached, keeps the music moving as
well as gives it what I think must have seemed to his audiences in
Paris in 1928 a foreign air -- the rhythm is Indian, perhaps, or
"magical," but not French.  It provides a constant forward drive which
propels the piece from beginning to end in a rock-solid tempo.

In measure five, a solo flute enters, carrying the A theme so
wistfully that the listener must listen carefully to hear it.  Such a
wistful tune, so extremely sensual that the @i[Bolero] has earned a
name for itself as a lovers' song (Listening to the piece, I can
completely understand why it is considered one of the best pieces ever
written to accompany love-making, but that's besides the point.).
This theme A, as I call it, consists of two parts, an a and a', each
of which is eight measures long (The entire A theme is found in Figure
B.).  After two transition measures, the A theme is repeated
identically (excepting a transposition) by a solo clarinet; this time,
the tambourine rhythm is accompanied by flute.

Two measures later, at measure 40, the bassoon enters, and this is the
first we hear of the B theme.  Similar to the A theme in structure,
the B theme is composed of a b and b', each of which is eight measures
long (The B theme is figure C.).  Following the sixteen measures of
the B theme and two transitional measures, we hear the B theme again,
identical but for transposition, in a soprano clarinet.

The pattern continues in this manner.... two instruments or groups of
instruments play the A theme one after the other, than two instruments
or groups play the B theme, each theme separated by two transitional
measure.  Each time, a few more instruments play, or a little more
volume is added, in order to make the piece constantly build.  This
happens for four full A A B B cycles, including the first cycle
described above.  The build is so gradual that the first and second
violins don't even join until the beginning of the third cycle, and
the strings do not perform at full volume with all strings playing
arco until the eighteenth theme presentation. i.e. halfway through the
fifth cycle.

But the fifth cycle is not like the others.  It begins loadly but
normally with a presentation of the A theme, but where we would expect
a repetition of the A theme, we instead hear an even louder
presentation of the B theme.  We feel at this point the approaching
resolution as, at the end of the B theme, the entire orchestra
suddenly modulates into E major from the C major key they have
maintained throughout the piece, remains there for a few measures and
then plunges back into the C major key to strike the final note of the
@i[Bolero] in one incredible moment of ecstasy.

@Heading(Conclusion)

By now you can probably tell that I did, indeed, enjoy this piece.
Several times while I was listening to the piece and watching Zubin
Mehta conduct it, I could not help but say to myself, "Brilliant!  The
man was brilliant!"  I don't know why, but no other piece (besides
possibly the @i[Brandenburg Concerti] of Bach) has struck the way this
one did when I first heard it.  I was immediately spellbound by the
power Ravel wielded, and by the beauty with which he wielded it.  So,
to answer the questions I asked myself at the beginning of this
paper...

I understand why this piece is different from other pieces I have
heard.  The repeated melody, the incredible blending of unique
instruments, the incredibly orchestrated dynamics, all these show me
how different the piece is from anything we've done up until now, and
I like it.  I think that I can now feel confident that I will
recognize pieces written in this genre, and therefore pieces written
in this time period, when I hear them in the future.

@Newpage()

@Heading(Bibliography)

@Begin(transparent, leftmargin +.5 inch, indent -.5 inch)
@u[The Bolero].  Pyramid Films, Santa Monica (1975) Videocasette.

Ravel, Maurice.  @u[Bolero], @u[La Valse], @u[Le Tombeau de Conperin],
@u[Pavane pour une infante d@eac()finte]. Concertgebouw Orchestra,
Amsterdam, Bernard Haitink conducting.  Amsterdam: Philips Recording
Studios.

Sadie, Stanley, ed.  @u[The New Grove Dictionary of Music and
Musicians].  London: Macmillan Publishers Limited (1980), vol. 15, pp.
609-621.

Schindler, Allan.  @u[Listening to Music].  New York: Holt, Rinehart
and Winston (1980), pp. 33, 237-238, 258.
@End(transparent)
