;;; -*- Package: OPS -*-

;;; 6.034 Problem Set 2
;;; Jonathan Kamens
;;; October 15, 1991

(in-package 'ops)

; Questions:
; 
; Is the monkey, standing on the couch, tall enough to reach the
; bananas on the ceiling?
; 
; If not, can the ladder be placed on the couch and climbed on top of?
; 
; If not, can the ladder and the couch occupy the same coordinates?
; 
; If not, how can the monkey get the bananas if they are directly
; above the couch?
; 
; I'm going to assume that objects are allowed to occupy the same
; coordinates.

;;; Problem 1

; The first problem is to determine a set of forms that can represent
; the world adequately.  We can take one of two approaches -- either
; have a separate form for each object and one for the monkey, or have
; a general "object" form with attributes that allow enough
; information to be specified to be able to tell what object is
; identified by an instantiation of the form.  I will use the latter
; approach.
; 
; Also, we can either indicate in an object's form whether or not it
; is being held, or indicate in the monkey's form what he is holding,
; or both.  I will do both, because it is not difficult to do both in
; rules, and because it'll make lhs writing easier.  There will be
; other redundant information in the various forms in the WM.

(literalize object
	    name		; name of object
	    on			; object this one is on or floor
	    location		; X-Y, or monkey if being carried
	    weight)		; light or heavy

(literalize monkey
	    on			; couch, ladder, or floor
	    location		; X-Y
	    holding)		; object the monkey is holding

;;; Problem 2

; Commented out because the augmented start-rule is now being used.

(if (= problem 2)
    (p start-rule {<starter> (start 1)} -->
       (make object ^name couch ^on floor ^location 1-6 ^weight heavy)
       (make object ^name ladder ^on floor ^location 9-4 ^weight light)
       (make object ^name bananas ^on ceiling ^location 3-3 ^weight light)
       (make monkey ^on couch ^location 1-6)
       (remove <starter>)))

; Example of (wm) after running this:
; 
; > (wm)
; 
; 2:  (OBJECT    ^ON FLOOR    ^LOCATION 1-6    ^NAME COUCH    ^WEIGHT HEAVY)
; 3:  (OBJECT    ^ON FLOOR    ^LOCATION 9-4    ^NAME LADDER    ^WEIGHT LIGHT)
; 4:  (OBJECT    ^ON CEILING    ^LOCATION 3-3    ^NAME BANANAS    ^WEIGHT LIGHT)
; 5:  (MONKEY    ^ON COUCH    ^LOCATION 1-6)
; NIL

;;; Problem 3

; Goal form

(if (> problem 2)
    (literalize goal
		status
		type
		object-name
		to))

(if (= problem 3)
    (p start-rule {<starter> (start 1)} -->
       (make object ^name couch ^on floor ^location 1-6 ^weight heavy)
       (make object ^name ladder ^on floor ^location 9-4 ^weight light)
       (make object ^name bananas ^on ceiling ^location 3-3 ^weight light)
       (make monkey ^on couch ^location 1-6)
       (make goal ^status active ^type holds ^object-name bananas)
       (remove <starter>)))

; I'm listing lhs items out when I could compress them somewhat
; because it's easier to verify correctness that way.

; The following goals are needed: holds an object, hands are empty, is
; on an object, is at a new location, is at a new location with an
; object

(p holds
   {<goal> (goal ^status active ^type holds ^object-name <o>)}
   (monkey ^holding <o>)
   -->
   (modify <goal> ^status satisfied))

(p empty
   {<goal> (goal ^status active ^type empty)}
   (monkey ^holding nil)
   -->
   (modify <goal> ^status satisfied))

(p on
   {<goal> (goal ^status active ^type on ^object-name <o>)}
   (monkey ^on <o>)
   -->
   (modify <goal> ^status satisfied))

(p already-there
   {<goal> (goal ^status active ^type walks-to ^to <l>)}
   (monkey ^location <l>)
   -->
   (modify <goal> ^status satisfied))

(p with-object
   {<goal> (goal ^status active ^type moves-to ^to <l> ^object-name <o>)}
   (monkey ^location <l>)
   (object ^name <o> ^location <l>)
   -->
   (modify <goal> ^status satisfied))

; If the monkey wants to hold something, he can't be holding anything
; else

(p empty-for-grab
   (goal ^status active ^type holds ^object-name <o>)
   (monkey ^holding <> <o> <> nil)
   -->
   (make goal ^status active ^type empty))

; If the monkey wants to hold something that isn't on the ceiling, he
; has to be at its location

(p walk-for-grab
   (goal ^status active ^type holds ^object-name <o>)
   (object ^name <o> ^on <> ceiling ^location {<l> <> monkey})
   -->
   (make goal ^status active ^type walks-to ^to <l>))

; If the monkey wants to hold something that isn't on the ceiling, he
; has to be on the floor

(p floor-for-grab
   (goal ^status active ^type holds ^object-name <o>)
   (object ^name <o> ^on <> ceiling)
   -->
   (make goal ^status active ^type on ^object-name <o>))

; If the monkey wants to hold something that is on the ceiling, he has
; to be at its location with the ladder

(p move-ladder-for-grab
   (goal ^status active ^type holds ^object-name <o>)
   (object ^name <o> ^on ceiling ^location {<l> <> monkey})
   -->
   (make goal ^status active ^type moves-to ^to <l> ^object-name ladder))

; The monkey has to get up onto the ladder to do the grabbing if the
; object is on the ceiling

(p climb-ladder-for-grab
   (goal ^status active ^type holds ^object-name <o>)
   (object ^name <o> ^on ceiling ^location <l>)
   (monkey ^location <l> ^on <> ladder)
   -->
   (make goal ^status active ^type on ^object-name ladder))

; Do the grabbing of an object not on the ceiling

(p grab-from-floor
   (goal ^status active ^type holds ^object-name <o>)
   {<object> (object ^name <o> ^on <> ceiling ^location <l> ^weight light)}
   {<monkey> (monkey ^holding nil ^on floor ^location <l>)}
   -->
   (modify <object> ^on nil ^location monkey)
   (modify <monkey> ^holding <o>)
   (write (crlf) grab <o>))

; Do the grabbing of an object on the ceiling

(p grab-on-ceiling
   (goal ^status active ^type holds ^object-name <o>)
   {<object> (object ^name <o> ^on ceiling ^location <l> ^weight light)}
   {<monkey> (monkey ^holding nil ^on ladder ^location <l>)}
   -->
   (modify <object> ^on nil ^location monkey)
   (modify <monkey> ^holding <o>)
   (write (crlf) grab <o>))

; The monkey empties his hands simply by getting rid of what he's
; holding.

(p do-empty
   (goal ^status active ^type empty)
   {<monkey> (monkey ^holding {<o> <> nil} ^on <o2> ^location <l>)}
   {<object> (object ^name <o>)}
   -->
   (modify <monkey> ^holding nil)
   (modify <object> ^on <o2> ^location <l>)
   (write (crlf) release <o>))

; The monkey gets onto the floor by getting off of what he's on

(p onto-floor
   (goal ^status active ^type on ^object-name floor)
   {<monkey> (monkey ^on <> floor)}
   -->
   (modify <monkey> ^on floor)
   (write (crlf) climb onto floor))

; The monkey needs to be onto the floor to climb onto anything else

(p floor-for-climb
   (goal ^status active ^type on ^object-name {<o> <> floor})
   (monkey ^on <> <o> <> floor)
   -->
   (make goal ^status active ^type on ^object-name floor))

; The monkey needs to be empty-handed to climb onto anything else

(if (< problem 5)
    (p empty-for-climb
       (goal ^status active ^type on ^object-name <> floor)
       (monkey ^holding <> nil)
       -->
       (make goal ^status active ^type empty))
  (p empty-for-climb
     (goal ^status active ^type on ^object-name <> floor)
     (monkey ^holding ladder)
     -->
     (make goal ^status active ^type empty)))


; The monkey needs to be at an object's location to climb onto it

(p walk-for-climb
   (goal ^status active ^type on ^object-name {<o> <> floor})
   (object ^name <o> ^location {<l> <> monkey})
   (monkey ^location <> <l>)
   -->
   (make goal ^status active ^type walks-to ^to <l>))

; Do the climb

(if (< problem 5)
    (p do-climb
       (goal ^status active ^type on ^object-name {<o> <> floor})
       (object ^name <o> ^location <l>)
       {<monkey> (monkey ^holding nil ^on floor ^location <l>)}
       -->
       (modify <monkey> ^on <o>)
       (write (crlf) climb onto <o>))
  (p do-climb
     (goal ^status active ^type on ^object-name {<o> <> floor})
     (object ^name <o> ^location <l>)
     {<monkey> (monkey ^holding <> ladder ^on floor ^location <l>)}
     -->
     (modify <monkey> ^on <o>)
     (write (crlf) climb onto <o>)))

; In order to walk to a new location, monkey has to be on the floor

(p floor-for-walk
   (goal ^status active ^type walks-to)
   (monkey ^on <> floor)
   -->
   (make goal ^status active ^type on ^object-name floor))

; Do the walk

(p do-walk
   (goal ^status active ^type walks-to ^to <l>)
   {<monkey> (monkey ^on floor)}
   -->
   (modify <monkey> ^location <l>)
   (write (crlf) walk to <l>))

; In order to move an object, monkey must be on the floor

(p floor-for-move
   (goal ^status active ^type moves-to)
   (monkey ^on <> floor)
   -->
   (make goal ^status active ^type on ^object-name floor))

; In order to move an object, monkey must have it

(p grab-for-move
   (goal ^status active ^type moves-to ^object-name <o>)
   (monkey ^holding <> <o>)
   -->
   (make goal ^status active ^type holds ^object-name <o>))

(p do-move
   (goal ^status active ^type moves-to ^to <l> ^object-name <o>)
   {<monkey> (monkey ^on floor ^holding <o> ^location <> <l>)}
   -->
   (modify <monkey> ^location <l>)
   (write (crlf) carry <o> to <l>)
   (make goal ^status active ^type empty))

; Sample output
; 
; > (run)
; 
; 2. START-RULE 11
; 3. EMPTY-FOR-GRAB 16 15
; 4. EMPTY 18 15
; 5. MOVE-LADDER-FOR-GRAB 16 14
; 6. FLOOR-FOR-MOVE 21 15
; 7. ONTO-FLOOR 22 15
; CLIMB ONTO FLOOR 
; 8. ON 22 24
; 9. GRAB-FOR-MOVE 21 24
; 10. EMPTY-FOR-GRAB 27 24
; 11. EMPTY 28 24
; 12. WALK-FOR-GRAB 27 13
; 13. DO-WALK 31 24
; WALK TO 9-4 
; 14. ALREADY-THERE 31 33
; 15. GRAB-FROM-FLOOR 27 13 33
; GRAB LADDER 
; 16. HOLDS 27 39
; 17. DO-MOVE 21 39
; CARRY LADDER TO 3-3 
; 18. DO-EMPTY 44 43 37
; RELEASE LADDER 
; 19. WITH-OBJECT 21 46 48
; 20. EMPTY 44 46
; 21. CLIMB-LADDER-FOR-GRAB 16 14 46
; 22. DO-CLIMB 53 48 46
; CLIMB ONTO LADDER 
; 23. ON 53 55
; 24. GRAB-ON-CEILING 16 14 55
; GRAB BANANAS 
; 25. HOLDS 16 61
; end -- no production true
;  24 productions (114 // 250 nodes)
;  25 firings (63 rhs actions)
;   9 mean working memory size (13 maximum)
;   3 mean conflict set size (7 maximum)
;  22 mean token memory size (27 maximum)
; NIL
; > 

;;; Problem 4

(if (eq problem 4)
    (p start-rule {<starter> (start 1)} -->
       (make object ^name couch ^on floor ^location 1-6 ^weight heavy)
       (make object ^name ladder ^on floor ^location 9-4 ^weight light)
       (make object ^name bananas ^on ceiling ^location 3-3 ^weight light)
       (make monkey ^on couch ^location 1-6)
       (make goal ^status active ^type moves-to ^object-name bananas ^to 7-7)
       (remove <starter>)))

; Sample output

; > (run)
; 
; 32. START-RULE 149
; 33. FLOOR-FOR-MOVE 154 153
; 34. ONTO-FLOOR 156 153
; CLIMB ONTO FLOOR 
; 35. ON 156 158
; 36. GRAB-FOR-MOVE 154 158
; 37. EMPTY-FOR-GRAB 161 158
; 38. EMPTY 162 158
; 39. MOVE-LADDER-FOR-GRAB 161 152
; 40. GRAB-FOR-MOVE 165 158
; 41. EMPTY-FOR-GRAB 166 158
; 42. EMPTY 167 158
; 43. WALK-FOR-GRAB 166 151
; 44. DO-WALK 170 158
; WALK TO 9-4 
; 45. ALREADY-THERE 170 172
; 46. GRAB-FROM-FLOOR 166 151 172
; GRAB LADDER 
; 47. HOLDS 166 178
; 48. DO-MOVE 165 178
; CARRY LADDER TO 3-3 
; 49. DO-EMPTY 183 182 176
; RELEASE LADDER 
; 50. WITH-OBJECT 165 185 187
; 51. EMPTY 183 185
; 52. CLIMB-LADDER-FOR-GRAB 161 152 185
; 53. DO-CLIMB 192 187 185
; CLIMB ONTO LADDER 
; 54. ON 192 194
; 55. GRAB-ON-CEILING 161 152 194
; GRAB BANANAS 
; 56. HOLDS 161 200
; 57. FLOOR-FOR-MOVE 154 200
; 58. ONTO-FLOOR 203 200
; CLIMB ONTO FLOOR 
; 59. ON 203 205
; 60. DO-MOVE 154 205
; CARRY BANANAS TO 7-7 
; 61. DO-EMPTY 210 209 198
; RELEASE BANANAS 
; 62. WITH-OBJECT 154 212 214
; 63. EMPTY 210 212
; end -- no production true
;  24 productions (116 // 256 nodes)
;  63 firings (218 rhs actions)
;  10 mean working memory size (24 maximum)
;   3 mean conflict set size (8 maximum)
;  22 mean token memory size (35 maximum)
; NIL
; > 

;;; Extra Credit Problem

; The problem is a bit under-specified, so some assumptions have to be
; made.  First of all, as the clarifications in
; /mit/6.034/ps2/ps2.clarify point out, you have to assume that the
; monkey can climb while carrying the bananas.  That means that
; do-climb and empty-for-climb have to be modified as indicated above
; so that they allow the monkey to be carrying anything but the ladder
; when doing the climb.
; 
; Second, you have to assume that if the monkey is on the ladder with
; the bananas, he can put them on the ceiling.

; If we want the bananas on the ceiling, we must put move them to
; where we're eventually going to hang them from

(p move-bananas-for-hang
   (goal ^status active ^type hang ^to <l>)
   - (monkey ^holding bananas)
   (object ^name bananas ^location <> <l>)
   -->
   (make goal ^status active ^type moves-to ^object-name bananas ^to <l>))

; If we want the bananas on the ceiling, we need to put the ladder in
; the right place, after moving the bananas

(p move-ladder-for-hang
   (goal ^status active ^type hang ^to <l>)
   (object ^name ladder ^location <> <l>)
   (object ^name bananas ^location <l>)
   -->
   (make goal ^status active ^type moves-to ^object-name ladder ^to <l>))

; If the ladder and bananas are in the right place, we need to pick up
; the bananas

(p get-bananas-for-hang
   (goal ^status active ^type hang ^to <l>)
   (object ^name ladder ^location <l>)
   (object ^name bananas ^location <l>)
   -->
   (make goal ^status active ^type holds ^object-name bananas))

; If we want the bananas on the ceiling and everything's in the right
; location, climb the ladder

(p climb-ladder-for-hang
   (goal ^status active ^type hang ^to <l>)
   (object ^name ladder ^location <l>)
   (monkey ^holding bananas)
   -->
   (make goal ^status active ^type on ^object-name ladder))

; When everything else is set, we can hang up the bananas

(p do-hang
   {<goal> (goal ^status active ^type hang ^to <l>)}
   {<object> (object ^name ladder ^location <l>)}
   {<monkey> (monkey ^holding bananas ^on ladder)}
   -->
   (modify <goal> ^status satisfied)
   (modify <object> ^on ceiling ^location <l>)
   (modify <monkey> ^holding nil)
   (write (crlf) hang bananas on ceiling))

(if (= problem 5)
    (p start-rule {<starter> (start 1)} -->
       (make object ^name couch ^on floor ^location 1-6 ^weight heavy)
       (make object ^name ladder ^on floor ^location 9-4 ^weight light)
       (make object ^name bananas ^on ceiling ^location 3-3 ^weight light)
       (make monkey ^on couch ^location 1-6)
       (make goal ^status active ^type hang ^to 7-7)
       (remove <starter>)))

; This problem is complicated by the fact that the two subgoals of
; getting the bananas to 7-7 and getting the ladder to 7-7 may
; interfere with each other -- getting the bananas might require
; moving the ladder.  I got around this problem by requiring the
; bananas to be moved into place before getting the ladder in the
; special rules above.
; 
; I had to change the climbing rules above (see do-climb and
; empty-for-climb) to allow climbing while carrying the bananas.
; 
; One more problem is that the OPS firer goes into an infinite loop
; when it is run on problem 5 with LEX strategy enabled.  It does,
; however, work with MEA strategy, as illustrated in the following
; output:

; > (run)
; 
; 146. START-RULE 425
; 147. MOVE-BANANAS-FOR-HANG 430 428
; 148. FLOOR-FOR-MOVE 432 429
; 149. ONTO-FLOOR 433 429
; CLIMB ONTO FLOOR 
; 150. ON 433 435
; 151. GRAB-FOR-MOVE 432 435
; 152. MOVE-LADDER-FOR-GRAB 438 428
; 153. GRAB-FOR-MOVE 439 435
; 154. WALK-FOR-GRAB 440 427
; 155. DO-WALK 441 435
; WALK TO 9-4 
; 156. ALREADY-THERE 441 443
; 157. GRAB-FROM-FLOOR 440 427 443
; GRAB LADDER 
; 158. HOLDS 440 449
; 159. DO-MOVE 439 449
; CARRY LADDER TO 3-3 
; 160. DO-EMPTY 454 453 447
; RELEASE LADDER 
; 161. EMPTY 454 456
; 162. WITH-OBJECT 439 456 458
; 163. CLIMB-LADDER-FOR-GRAB 438 428 456
; 164. DO-CLIMB 463 458 456
; CLIMB ONTO LADDER 
; 165. ON 463 465
; 166. GRAB-ON-CEILING 438 428 465
; GRAB BANANAS 
; 167. HOLDS 438 471
; 168. FLOOR-FOR-MOVE 432 471
; 169. ONTO-FLOOR 474 471
; CLIMB ONTO FLOOR 
; 170. ON 474 476
; 171. DO-MOVE 432 476
; CARRY BANANAS TO 7-7 
; 172. DO-EMPTY 481 480 469
; RELEASE BANANAS 
; 173. EMPTY 481 483
; 174. WITH-OBJECT 432 483 485
; 175. MOVE-LADDER-FOR-HANG 430 458 485
; 176. GRAB-FOR-MOVE 490 483
; 177. WALK-FOR-GRAB 491 458
; 178. DO-WALK 492 483
; WALK TO 3-3 
; 179. ALREADY-THERE 492 494
; 180. GRAB-FROM-FLOOR 491 458 494
; GRAB LADDER 
; 181. HOLDS 491 500
; 182. DO-MOVE 490 500
; CARRY LADDER TO 7-7 
; 183. DO-EMPTY 505 504 498
; RELEASE LADDER 
; 184. EMPTY 505 507
; 185. WITH-OBJECT 490 507 509
; 186. GET-BANANAS-FOR-HANG 430 509 485
; 187. GRAB-FROM-FLOOR 514 485 507
; GRAB BANANAS 
; 188. HOLDS 514 518
; 189. CLIMB-LADDER-FOR-HANG 430 509 518
; 190. DO-CLIMB 521 509 518
; CLIMB ONTO LADDER 
; 191. ON 521 523
; 192. DO-HANG 430 509 523
; HANG BANANAS ON CEILING 
; end -- no production true
;  16 productions (156 // 350 nodes)
; 192 firings (531 rhs actions)
;   9 mean working memory size (17 maximum)
;   3 mean conflict set size (6 maximum)
;  30 mean token memory size (39 maximum)
; NIL
; > 

In fact, it would probably have made sense to do the entire problem
set with MEA enabled, since the goal is the first condition in every
rule, and we pretty much always want the most recently activated goal
to be the one we're working on until we're finished with it.
