;;; -*- Package: USER -*-

;;; Jonathan Kamens
;;; 6.034 Problem Set 3

(in-package 'user)

;;; Problem 5

;; (a)

(setf noone-seated (make-instance 
		    'scientist-seating-state
		    :unseated (mapcar 'car *languages*)
		    :empty-seats *seats*
		    :seating '()))

(setf search-instance (make-instance 'search-method))

; The daughters of the scientist-seating-state try to seat each unseated
; scientist in each empty seat.

; We don't really have to check for incompatible states here, but I think
; that checking for it here saves us from a lot of effort later.

(defmethod daughters ((s scientist-seating-state))
  (if (seating s)
      (let ((new-states '()) new-state)
	(dolist (name (unseated s))
	  (setq new-state
		(make-instance 
		 (get-argument-class s)
		 :unseated (remove name (unseated s))
		 :empty-seats (rest (empty-seats s))
		 :seating (acons name (first (empty-seats s)) (seating s))))
	  (if (not (incompatible? new-state))
	      (setq new-states (cons new-state new-states))))
	new-states)
    (list (make-instance
	   (get-argument-class s)
	   :unseated (rest (unseated s))
	   :empty-seats (rest (empty-seats s))
	   :seating (acons (first (unseated s)) (first (empty-seats s))
			   '())))))

; Here is the log of running (do-search search-instance noone-seated):
; 
; > (do-search search-instance noone-seated)
; Empty Seats:  (0 1 2 3 4 5)    Unseated:  (Sushiblzjknikov Minimanetmonet Petitsakistein Skjzrnerfuss Saladressing Glasnostrudel)
;      Seated:  ()
; Value: 0
; Empty Seats:  (1 2 3 4 5)      Unseated:  (Minimanetmonet Petitsakistein Skjzrnerfuss Saladressing Glasnostrudel)
;      Seated:  ((Sushiblzjknikov . 0))
; Value: 0
; Empty Seats:  (2 3 4 5)        Unseated:  (Minimanetmonet Petitsakistein Skjzrnerfuss Saladressing)
;      Seated:  ((Glasnostrudel . 1) (Sushiblzjknikov . 0))
; Value: 1
; Empty Seats:  (3 4 5)          Unseated:  (Minimanetmonet Petitsakistein Skjzrnerfuss)
;      Seated:  ((Saladressing . 2) (Glasnostrudel . 1) (Sushiblzjknikov . 0))
; Value: 1
; Empty Seats:  (4 5)            Unseated:  (Minimanetmonet Petitsakistein)
;      Seated:  ((Skjzrnerfuss . 3) (Saladressing . 2) (Glasnostrudel . 1) (Sushiblzjknikov . 0))
; Value: 2
; Empty Seats:  (5)              Unseated:  (Minimanetmonet)
;      Seated:  ((Petitsakistein . 4) (Skjzrnerfuss . 3) (Saladressing . 2) (Glasnostrudel . 1) (Sushiblzjknikov . 0))
; Value: 3
; Empty Seats:  ()               Unseated:  ()
;      Seated:  ((Minimanetmonet . 5) (Petitsakistein . 4) (Skjzrnerfuss . 3) (Saladressing . 2) (Glasnostrudel . 1) (Sushiblzjknikov . 0))
; Value: 4
; #:|instance4112|
; > 

;; (b)

; Count the total number of languages being spoken in the current state.
; 
; Special case -- if there is an incompatible seat anywhere in the current
; state, the value is very high, higher than it would otherwise be, to prevent
; any further searching.
; 
; Assumes that any two scientists have one language at most in common, which 
; is true in the current state space.

(defun calc-value (s)
  (if (incompatible? s)
      most-positive-fixnum
    (let ((cost 0) (languages '()) templang)
      (dolist (i (mapcar 'cdr (seating s)))
	(if (someone-at? s (clockwise i))
	    (progn
	      (setq templang
		    (car (common-language? (person-at s i) 
					   (person-at s (clockwise i)))))
	      (if (not (assoc templang languages))
		  (setq languages (cons (cons templang t) languages)))))
	(if (someone-at? s (counterclockwise i))
	    (progn
	      (setq templang
		    (car (common-language? (person-at s i) 
					   (person-at s
						      (counterclockwise i)))))
	      (if (not (assoc templang languages))
		  (setq languages (cons (cons templang t) languages))))))
      (dolist (i languages)
	(setq cost (1+ cost)))
      cost)))

;; PRINT-STATE is called by continue-search when *verbose* is true;
;; you may want to specialize or redefine this for your own tracing needs.
(defmethod PRINT-STATE ((s scientist-seating-state))
  (format t "~&Empty Seats:  ~15@<~:a~>" (empty-seats s))
  (format t "  Unseated:  ~:(~:a~)" (unseated s))
  (format t "~&     Seated:  ~:(~:a~)" (seating s))
  (format t "~&Value: ~s" (value s)))

(defclass best-first-search (search-method))

(defmethod add-states ((m best-first-search) new-states)
  (setq new-states (sort new-states #'< :key #'value))
  (setf (queue m) (merge 'list new-states (queue m) #'< :key #'value)))
 
(setf search-instance2 (make-instance 'best-first-search))

; It isn't really necessary to define a new class for the valued
; scientist seating state, but doing so allows us to memoize value 
; calculations, which speeds up things considerably.

(defclass valued-scientist-seating-state (scientist-seating-state)
  ((real-value :accessor memoized-value :initform nil)))

(setf noone-seated2 (make-instance 
		    'valued-scientist-seating-state
		    :unseated (mapcar 'car *languages*)
		    :empty-seats *seats*
		    :seating '()))

(defmethod value ((s scientist-seating-state))
  (calc-value s))

(defmethod value ((s valued-scientist-seating-state))
  (if (not (null (memoized-value s)))
      (memoized-value s)
    (setf (memoized-value s) (calc-value s))))

; I don't see how I'm supposed to turn in a "short session log illustrating
; the application of these functions in part ... (b)," since the sequence of
; steps that are taken in the execution of (do-search search-instance2
; noone-seated2) is several pages long.  Instead, I'll describe what happens
; and include the last part of the log.  What it does is first try all of the
; states with value 0 (there are only two -- with zero people seated and with
; one person seated), then all of the states with value 1, then all of the
; states with value 2, etc.  The final solution ends up having a value of 3,
; so here is that portion of the log:
; 
; Value: 3
; Empty Seats:  (5)              Unseated:  (Petitsakistein)
;      Seated:  ((Skjzrnerfuss . 4) (Saladressing . 3) (Minimanetmonet . 2) (Glasnostrudel . 1) (Sushiblzjknikov . 0))
; Value: 3
; Empty Seats:  (5)              Unseated:  (Skjzrnerfuss)
;      Seated:  ((Petitsakistein . 4) (Saladressing . 3) (Minimanetmonet . 2) (Glasnostrudel . 1) (Sushiblzjknikov . 0))
; Value: 3
; Empty Seats:  (4 5)            Unseated:  (Skjzrnerfuss Saladressing)
;      Seated:  ((Petitsakistein . 3) (Minimanetmonet . 2) (Glasnostrudel . 1) (Sushiblzjknikov . 0))
; Value: 3
; Empty Seats:  (5)              Unseated:  (Skjzrnerfuss)
;      Seated:  ((Saladressing . 4) (Petitsakistein . 3) (Minimanetmonet . 2) (Glasnostrudel . 1) (Sushiblzjknikov . 0))
; Value: 3
; Empty Seats:  (5)              Unseated:  (Minimanetmonet)
;      Seated:  ((Saladressing . 4) (Skjzrnerfuss . 3) (Petitsakistein . 2) (Glasnostrudel . 1) (Sushiblzjknikov . 0))
; Value: 3
; Empty Seats:  (5)              Unseated:  (Saladressing)
;      Seated:  ((Minimanetmonet . 4) (Skjzrnerfuss . 3) (Petitsakistein . 2) (Glasnostrudel . 1) (Sushiblzjknikov . 0))
; Value: 3
; Empty Seats:  (4 5)            Unseated:  (Minimanetmonet Skjzrnerfuss)
;      Seated:  ((Saladressing . 3) (Petitsakistein . 2) (Glasnostrudel . 1) (Sushiblzjknikov . 0))
; Value: 3
; Empty Seats:  (5)              Unseated:  (Skjzrnerfuss)
;      Seated:  ((Minimanetmonet . 4) (Saladressing . 3) (Petitsakistein . 2) (Glasnostrudel . 1) (Sushiblzjknikov . 0))
; Value: 3
; Empty Seats:  (4 5)            Unseated:  (Skjzrnerfuss Saladressing)
;      Seated:  ((Minimanetmonet . 3) (Petitsakistein . 2) (Glasnostrudel . 1) (Sushiblzjknikov . 0))
; Value: 3
; Empty Seats:  (5)              Unseated:  (Skjzrnerfuss)
;      Seated:  ((Saladressing . 4) (Minimanetmonet . 3) (Petitsakistein . 2) (Glasnostrudel . 1) (Sushiblzjknikov . 0))
; Value: 3
; Empty Seats:  (5)              Unseated:  (Minimanetmonet)
;      Seated:  ((Saladressing . 4) (Petitsakistein . 3) (Skjzrnerfuss . 2) (Glasnostrudel . 1) (Sushiblzjknikov . 0))
; Value: 3
; Empty Seats:  (5)              Unseated:  (Saladressing)
;      Seated:  ((Minimanetmonet . 4) (Petitsakistein . 3) (Skjzrnerfuss . 2) (Glasnostrudel . 1) (Sushiblzjknikov . 0))
; Value: 3
; Empty Seats:  ()               Unseated:  ()
;      Seated:  ((Saladressing . 5) (Minimanetmonet . 4) (Petitsakistein . 3) (Skjzrnerfuss . 2) (Glasnostrudel . 1) (Sushiblzjknikov . 0))
; Value: 3
; #:|instance3794|
; > 

;; (c)

; I can't do this.  It's too complicated, and I don't have time.

;; Comparison for part (b)

; The sequence of steps followed in part (b) is obviously very different
; from the steps followed in (a).  In (b), the search first looks at all of
; the tree nodes using 0 languages, then at the nodes using 1 language, then
; at the nodes using 2 languages, etc.  It eventually finds a solution node
; using 3 languages.  In part (a), the search simply goes down the three in
; a depth-first fashion.  It finds a solution, and it does it much faster than
; the search in (b), but the solution uses 4 languages instead of 3.
