;;; -*- Mode: Lisp; package: user -*- 

(in-package 'user)

(defconstant *time-labels*
  '(b m o s d f b- m- o- s- d- f- =))

(defvar *time-constraint-table*
  '(
    (b
     (b b)
     (m b)
     (o b)
     (s b)
     (d b m o s d)
     (f b m- o- s- d-)
     (b- b m o s d f b- m- o- s- d- f- =)
     (m- b m- o- s- d-)
     (o- b m- o- s- d-)
     (s- b)
     (d- b)
     (f- b)
     (= b))
    (m
     (b b)
     (m b)
     (o b)
     (s m)
     (d o s d)
     (f b m o s d f b- m- o- s- d- f- =)
     (b- b- m- o- s- d-)
     (m- f f- =)
     (o- o s d)
     (s- m)
     (d- b)
     (f- b)
     (= m))
    (o
     (b b)
     (m b)
     (o b m o)
     (s o)
     (d o s d)
     (f o s d)
     (b- b- m- o- s- d-)
     (m- o- s- d-)
     (o- o d o- d-)
     (s- o d- f-)
     (d- b m o d- f-)
     (f- b m o)
     (= o))
    (s
     (b b)
     (m b)
     (o b m o)
     (s s)
     (d d)
     (f d)
     (b- b-)
     (m- m-)
     (o- d d- f-)
     (s- s s- =)
     (d- b- m o d- f-)
     (f- b- m o)
     (= s))
    (d
     (b b)
     (m b)
     (o b m o s d)
     (s d)
     (d d)
     (f d)
     (b- b-)
     (m- b-)
     (o- d f b- m- o-)
     (s- d f b- m- o-)
     (d- o- d- o d f- f =)
     (f- b m o s d)
     (= d))
    (f
     (b b)
     (m m)
     (o o s d)
     (s d)
     (d d)
     (f f)
     (b- b-)
     (m- b-)
     (o- b- m- o-)
     (s- b- m- o-)
     (d- b- m- o- s- d-)
     (f- f f- =)
     (= f))
    (b-
     (b b m o s d f b- m- o- s- d- f- =)
     (m d f b- m- o-)
     (o d f b- m- o-)
     (s d f b- m- o-)
     (d d f b- m- o-)
     (f b-)
     (b- b-)
     (m- b-)
     (o- b-)
     (s- b-)
     (d- b-)
     (f- b-)
     (= b-))
    (m-
     (b b m d- f-)
     (m s s- =)
     (o d f o-)
     (s b-)
     (d d f o-)
     (f m-)
     (b- b-)
     (m- b-)
     (o- d f o-)
     (s- d f o-)
     (d- b-)
     (f- m-)
     (= m-))
    (o-
     (b b m o d- f-)
     (m o d- f-)
     (o o d o- d-)
     (s b-)
     (d d f o-)
     (f o-)
     (b- b-)
     (m- b-)
     (o- b- m o-)
     (s- b- m- o-)
     (d- b- m o- s- d-)
     (f- o- s- d-)
     (= o-))
    (s-
     (b b m o d- f-)
     (m o d- f-)
     (o o d- f-)
     (s s s- =)
     (d d f o)
     (f o-)
     (b- b-)
     (m- m-)
     (o- o-)
     (s- s-)
     (d- d-)
     (f- d-)
     (= s-))
    (d-
     (b b m o d- f-)
     (m o d- f-)
     (o o d- f-)
     (s o d- f-)
     (d o d f f- o- d- =)
     (f o d- f-)
     (b- b- m-)
     (m- o- s- d-)
     (o- o- s- d-)
     (s- d-)
     (d- d-)
     (f- d)
     (= d-))
    (f-
     (b b)
     (m m)
     (o o)
     (s o)
     (d d-)
     (f f f- =)
     (b- b- m- o- s- d-)
     (m- o- s- d-)
     (o- o- s- d-)
     (s- d-)
     (d- d-)
     (f- f-)
     (= f-))
    (=
     (b b)
     (m m)
     (o o)
     (s s)
     (d d)
     (f f)
     (b- b-)
     (m- m-)
     (o- o-)
     (s- s-)
     (d- d-)
     (f- f-)
     (= =))
    ))

(defun time-constraint-valid? (x y z)
  (let ((x-table (assoc x *time-constraint-table*)))
    (when (null x-table)
      (error "~s is not a valid time-constraint symbol" x))
    (let ((y-list (assoc y (cdr x-table))))
      (when (null y-list)
        (error "~s is not a valid time-constraint symbol" y))
      (member z (cdr y-list)))))

(defun make-time-relation-variable (name &optional (label-set *time-labels*))
  (let ((bad-labels (set-difference label-set *time-labels*)))
    (if bad-labels
	(error "***Invalid time labels ~s" bad-labels)
      (make-instance 'domain-variable
		     :name name
		     :domain label-set))))

;;; For (require 'time)
(provide 'time)
