#include <matrix.h>
#include <twodtransform.h>

/*
 * From the foot frame into the global frame
 */
Matrix L10(double x, double y, double angle)
{
     Matrix transformer = 0, rotater, translater;
     
     rotater = twod_degree_rotater(angle);
     translater = twod_translater(x, y);

     matrix_multiply(rotater, translater, &transformer);
     
     matrix_destroy(&rotater);
     matrix_destroy(&translater);

     return(transformer);
}

/*
 * From the leg frame into the foot frame
 */
Matrix L21(double angle)
{
     Matrix rotater, transformer = 0;
     static Matrix translater = 0;
     
     if (! translater) {
	  translater = twod_translater(0.5, 16.0);
     }
     
     rotater = twod_degree_rotater(angle);

     matrix_multiply(translater, rotater, &transformer);

     matrix_destroy(&rotater);

     return(transformer);
}

/*
 * From the body frame to the leg frame
 *
 * They use the same origin, so only rotation is necessary
 */
Matrix L32(double angle)
{
     return(twod_degree_rotater(angle));
}


/*
 * It's easier to draw the backwards somersault if we treat the body
 * as the lowest link, rather than treating the foot as the lowest link
 */

/*
 * From body frame to the global frame
 */
Matrix L30(double x, double y, double angle)
{
     Matrix transformer = 0, rotater, translater;
     
     rotater = twod_degree_rotater(angle);

     translater = twod_translater(x, y);

     matrix_multiply(rotater, translater, &transformer);
     
     matrix_destroy(&rotater);
     matrix_destroy(&translater);

     return(transformer);
}

/*
 * From leg frame to body frame
 *
 * They use the same origin, so only rotation is necessary
 */
Matrix L23(double angle)
{
     return(twod_degree_rotater(angle));
}


/*
 * From foot frame to leg frame
 */
Matrix L12(double angle)
{
     Matrix rotater, transformer = 0;
     static Matrix translater = 0;
     
     if (! translater) {
	  translater = twod_translater(-0.5, -16.0);
     }
     
     rotater = twod_degree_rotater(angle);

     matrix_multiply(rotater, translater, &transformer);

     matrix_destroy(&rotater);

     return(transformer);
}
