#include <assert.h>
#include "polynomial.h"

extern char *malloc();
extern int free();

Polynomial poly_create(int degree)
{
     Polynomial poly;
     double *ptr;
     
     poly.degree = degree;
     ptr = poly.coefs = (double *)
	  malloc((unsigned) (sizeof(double) * (degree + 1)));

     assert(ptr);

     for (degree++; degree; degree--)
	  *ptr++ = 0.0;

     return(poly);
}


void poly_destroy(Polynomial poly)
{
     assert(poly.coefs);

     free((char *) poly.coefs);

     return;
}


Polynomial poly_multiply(Polynomial poly1, Polynomial poly2)
{
     Polynomial prod;
     int i, j;
     
     prod = poly_create(poly1.degree + poly2.degree);

     for (i = 0; i <= poly1.degree; i++)
	  for (j = 0; j <= poly2.degree; j++)
	       prod.coefs[i + j] += poly1.coefs[i] * poly2.coefs[j];

     return(prod);
}


double poly_evaluate(Polynomial poly, double x)
{
     int i;
     double val = 0.0;
     double mul = 1.0;
     
     for (i = 0; i <= poly.degree; i++) {
	  val += poly.coefs[i] * mul;
	  mul *= x;
     }

     return(val);
}


Polynomial poly_scale(Polynomial poly, double scale)
{
     Polynomial result;
     int i;

     result = poly_create(poly.degree);

     for (i = 0; i <= poly.degree; i++)
	  result.coefs[i] = poly.coefs[i] * scale;

     return(result);
}


Polynomial poly_add(Polynomial poly1, Polynomial poly2)
{
     Polynomial result;
     int i;

     result = poly_create(poly.degree);

     for (i = 0; i < poly.degree; i++)
	  result.coefs[i] = poly1.coefs[i] + poly2.coefs[i];

     return(result);
}
